"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var slack_messages_1 = require("@atomist/slack-messages");
var node_fetch_1 = __importDefault(require("node-fetch"));
var url_join_1 = __importDefault(require("url-join"));
var sanitizeMarkdown = function (markdown) {
    return slack_messages_1.githubToSlack(markdown)
        .split('\n')
        .map(function (line) {
        // Strip out the ### prefix and replace it with *<word>* to make it bold
        if (line.startsWith('#')) {
            return "*" + line.replace(/^[#]+/, '') + "*";
        }
        return line;
    })
        .join('\n');
};
var SlackPlugin = /** @class */ (function () {
    function SlackPlugin(options) {
        this.name = 'Slack';
        this.options = options;
    }
    SlackPlugin.prototype.apply = function (auto) {
        var _this = this;
        auto.hooks.afterRelease.tapPromise(this.name, function (newVersion, commits, releaseNotes) { return __awaiter(_this, void 0, void 0, function () {
            var head, isSkipped;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!newVersion) {
                            return [2 /*return*/];
                        }
                        if ('dryRun' in auto.args && auto.args.dryRun) {
                            return [2 /*return*/];
                        }
                        head = commits[0];
                        if (!head) {
                            return [2 /*return*/];
                        }
                        isSkipped = head.labels.find(function (label) {
                            return auto.release.options.skipReleaseLabels.includes(label);
                        });
                        if (isSkipped) {
                            return [2 /*return*/];
                        }
                        if (!this.options.url) {
                            throw new Error('Slack url must be set to post a message to slack.');
                        }
                        return [4 /*yield*/, this.postToSlack(auto, newVersion, releaseNotes)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    };
    SlackPlugin.prototype.postToSlack = function (auto, newVersion, releaseNotes) {
        return __awaiter(this, void 0, void 0, function () {
            var project, body, token, releaseUrl;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!auto.git) {
                            return [2 /*return*/];
                        }
                        auto.logger.verbose.info('Posting release notes to slack.');
                        return [4 /*yield*/, auto.git.getProject()];
                    case 1:
                        project = _a.sent();
                        body = sanitizeMarkdown(releaseNotes);
                        token = process.env.SLACK_TOKEN;
                        releaseUrl = url_join_1.default(project.html_url, auto.git.options.owner, auto.git.options.repo, 'releases/tag', newVersion);
                        if (!token) {
                            auto.logger.verbose.warn('Slack may need a token to send a message');
                        }
                        return [4 /*yield*/, node_fetch_1.default("" + this.options.url + (token ? "?token=" + token : ''), {
                                method: 'POST',
                                body: JSON.stringify({
                                    text: [
                                        "@channel: New release *<" + releaseUrl + "|" + newVersion + ">*",
                                        body
                                    ].join('\n'),
                                    // eslint-disable-next-line camelcase
                                    link_names: 1
                                }),
                                headers: { 'Content-Type': 'application/json' }
                            })];
                    case 2:
                        _a.sent();
                        auto.logger.verbose.info('Posted release notes to slack.');
                        return [2 /*return*/];
                }
            });
        });
    };
    return SlackPlugin;
}());
exports.default = SlackPlugin;
//# sourceMappingURL=index.js.map